/*
			     GMIDEMO.C

  After compilation it must be linked with SMSCXHR.LIB (small model)
  or LMSCXHR.LIB (large model) to get GMIDEMO.EXE.

07.11.89 First implementation.................................Stefan Kriebel
07.12.89 Switch to GEMINI from GVL............................Stefan Kriebel
20.12.89 Break on <ESC>, next demo with any other key.........Stefan Kriebel
11.01.90 Insert Cake demo.....................................Stefan Kriebel
25.01.90 Clear screen for GEMINI174 bug fix...................Stefan Kriebel
15.05.90 Test EGOS loaded.....................................Stefan Kriebel
05.08.92 Longer delays between two zooms......................SK
28.10.92 Zoom only on boards without any flicker..............SK
*/

#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <signal.h>
#include <fcntl.h>
#include <time.h>
#include <math.h>
#include <stdlib.h>
#include <process.h>

#define TRUE	1
#define FALSE	0
#define ESCAPE 0x1b
#define MIN_REV	1*256+6
#define MAX_RAND 0x7fff
#define MILLISEC 1000
#define DEFAULT_INT 0xcc    /* default EGOS interrupt number */
#define OFFSET 0x20         /* offset to egos driver magic number */

#define MAXPRESENT 2.0
#define MAXRECTS 5000
#define MAXSWEEP 5
#define MAXCIRC 10
#define MAXELPS 20
#define MAXAREA 3.0
#define MAXZOOM 5
#define MAXCAKE 4.0

#define XHR_Black         0
#define XHR_Blue          1
#define XHR_Green         2
#define XHR_Cyan          3
#define XHR_Red           4
#define XHR_Magenta       5
#define XHR_Brown         6
#define XHR_LightGray     7
#define XHR_DarkGrey      8
#define XHR_LightBlue     9
#define XHR_LightGreen   10
#define XHR_LightCyan    11
#define XHR_LightRed     12
#define XHR_LightMagenta 13
#define XHR_LightYellow  14
#define XHR_LightWhite   15

/* ---- XHR - boardtypes ----*/

#define XHR_XHR           0
#define XHR_PCD           1
#define XHR_XXHR8         2
#define XHR_XXHR16        3
#define XHR_SPECTRA8      6
#define XHR_SPECTRA16     7
#define XHR_ORION8        8
#define XHR_ORION16       9
#define XHR_ALPHA        11
#define XHR_OMEGA8       12
#define XHR_OMEGA16      13
#define GEMINI644        0x100
#define GEMINI114        0x102
#define GEMINI174        0x104
#define GEMINI118        0x106
#define GEMINI118E       0x108
#define GEMINI858        0x10a
#define GEMINI158        0x10c
#define GEMINI178        0x10e
#define GEMINI854        0x110
#define GEMINI154        0x112

typedef unsigned char  u_char;
typedef unsigned short u_short;
typedef unsigned int   u_int;

typedef u_int XHR_addr;
typedef u_int XHR_color;
typedef int   XHR_coord;
typedef u_int XHR_DN;
typedef int   XHR_mode;
typedef u_int XHR_zoom;
typedef u_int XHR_dir;
typedef int   XHR_mask;
typedef u_int XHR_index;

#define RANDOM(limit) (rand() % (limit))

static u_short quit;
static XHR_coord x_width, y_width, x_cent, y_cent;
static char c;
static int count;

void init( void );
void presents( void );
void circles( void );
void ellipses( void );
void sweep( void );
void BaseZoom( void );
void FilledRects( void );
void FilledArea( void );
void Cake( void );
int delay( float );

void ctrlc()                         /* Control-C interrupt */
{
  signal(SIGINT, SIG_IGN);

  xhrini((unsigned short far *)1);   /* Switch back to VGA if GVL option */
  exit(0);                           /*    installed */
}

void ctrlc_init()                    /* Install Control-C interrupt */
{
  if (signal(SIGINT,ctrlc) == (int(*)())-1) {
    fprintf(stderr,"Cannot initialize CTRL-C handler");
    exit(1);
  }
   }

/********************************************************************
  Test whether EGOS driver loaded
  OFFSET bytes behind the EGOS interrupt entry
  must be the EGOS magic number 0xaaaa5555
********************************************************************/
int egos_loaded(int n)
{
  long i;
  long far *p;
  #pragma check_pointer(off)
  p = (long far *) (unsigned long) (n * 4);
  i=*p;
  p=(long far *)i;
  (long)p+=OFFSET;
  i=*p;
  #pragma check_pointer()
  if(i==0xaaaa5555L)
    return(1);
  else
    return(0);
/* here the same procedure in short form :
   return((*(*(long far * far *)((long)interrupt_number << 2)
	    + OFFSET/sizeof(long))==0xaaaa5555L)) ? 1 : 0);
*/

}

/*****************************************
  delay(time)
  Delay time seconds and return 0 or
  break and return 1 if any key pressed
*****************************************/
int delay(float time)
{
  long far *p = (long far *)0x46cL;   /* Use system tick (18.2 Hz) */
  long i;
  volatile long j;

  i=j=*p;
  while( ( j - i ) < (long)( 18.2 * time ) )
  { j = *p;
    if(kbhit())
    { quit=(getch()==ESCAPE);
      return(1);
    }
  }
  return(0);
}

/***********************************************
  int convert(char *)

  Converts ascii hex number to integer
***********************************************/
int convert(char *s)
{ int i,j,c;

  for(i=j=0;i<strlen(s);i++)
  { c=toupper(s[i]);
    if(c >= '0' && c <= '9')
      j = (j<<4) + c - '0';
    if(c >= 'A' && c <= 'F')
      j = (j<<4) + c - 'A'+ 10;
  }
  return( j );
}

int main(int argc,char **argv)
{
  int intnum;

  printf("\nELSA XHR Demo Program");
  /* get optional EGOS interrupt number  */
  if(argc==2)
    intnum=convert(argv[1]);
  else
    intnum= DEFAULT_INT;

  /* test if EGOS loaded */
  if(!egos_loaded(intnum))
  { printf("\nInstall EGOS before starting Demo.");
    return( 1 );
  }
  ctrlc_init();                     /* Install 'Control C' interrupt */

  printf("\n\npress any key for next picture or Esc to cancel");

  xhrini((unsigned short far *)2);  /* switch to GEMINI (with GVL option) */
  init();
  while(!quit)
  {
    if (!quit) presents();    delay(0.05);
    if (!quit) circles();     delay(0.05);
    if (!quit) ellipses();    delay(0.05);
    if (!quit) sweep();       delay(0.05);
    if (!quit) BaseZoom();    delay(0.05);
    if (!quit) FilledArea();  delay(0.05);
    if (!quit) FilledRects(); delay(0.05);
    if (!quit) Cake();        delay(0.05);
  }

  xhrini((unsigned short far *)1);
  printf("\n");
  return( 0 );
}

/*******************************
 *                             *
 *         INIT                *
 *                             *
 *******************************

   Initializes XHR and assigns memory to XHR.
   Defines the parameters (origin, colors ..) of each screen. */

void init()
{
  long now;

  xhrini((unsigned short far *)0);
  quit = FALSE;

  fscrn(XHR_Black);
  x_width = rdini(7);
  y_width = rdini(8);

  defar(0,0,x_width-1,y_width-1);      /* define protected area */
  armod(2);                            /* protected mode ON */
  x_cent = x_width/2;                  /* calc. center of basescreen */
  y_cent = y_width/2;

  srand((int)time(&now) %37);
};

/*******************************
 *                             *
 *         StringOut           *
 *                             *
 *******************************/

/* draws a string of characters at CP-position */

void stringout (xtext,chrzoom)
char *xtext;
XHR_zoom chrzoom;
{
  while (*xtext)
    chrct(*xtext++,chrzoom,chrzoom,0);
}

/*******************************
 *                             *
 *         ELSA LOGO           *
 *                             *
 *******************************/

/* draws the ELSA-Logo at CP's Position with variable ZoomFactor */

static int posx, posy;
double LogoZoom;

int X (int W);
int Y (int W);

int X (int W)
{
    return (posx+(int ) /* floor */ (LogoZoom*W));
}

int Y (int W)
{
    return (posy+(int ) /* floor */ (LogoZoom*W));
}


void ELSA (col,ZF)
XHR_color col;
double ZF;
{
  LogoZoom = ZF;

  posx = getcpx();
  posy = getcpy();
  colors(col,col);

  /***** Drawing of frame *****/

  amove(X(0),Y(2));
  aline(X(0),Y(33));
  aarc(X(2),Y(33),X(2),Y(35),1);
  aline(X(106),Y(35));
  aarc(X(106),Y(33),X(108),Y(33),1);
  aline(X(108),Y(2));
  aarc(X(106),Y(2),X(106),Y(0),1);
  aline(X(2),Y(0));
  aarc(X(2),Y(2),X(0),Y(2),1);

  amove(X(4),Y(3));
  aline(X(4),Y(32));
  aarc(X(6),Y(32),X(6),Y(34),1);
  aline(X(102),Y(34));
  aarc(X(102),Y(32),X(104),Y(32),1);
  aline(X(104),Y(3));
  aarc(X(102),Y(3),X(102),Y(1),1);
  aline(X(6),Y(1));
  aarc(X(6),Y(3),X(4),Y(3),1);

  amove(X(2),Y(2));
  paint(col,0);
  amove(X(106),Y(2));
  paint(col,0);

  /***** Drawing of letter 'E' *****/

  amove(X(7),Y(6));
  aline(X(7),Y(29));
  aarc(X(9),Y(29),X(9),Y(31),1);
  aline(X(29),Y(31));
  aline(X(29),Y(30));
  aline(X(13),Y(30));
  aarc(X(13),Y(28),X(11),Y(28),0);
  aline(X(11),Y(18));
  aline(X(29),Y(18));
  aline(X(29),Y(17));
  aline(X(11),Y(17));
  aline(X(11),Y(7));
  aarc(X(13),Y(7),X(13),Y(5),0);
  aline(X(29),Y(5));
  aline(X(29),Y(4));
  aline(X(9),Y(4));
  aarc(X(9),Y(6),X(7),Y(6),1);
  amove(X(9),Y(6));
  paint(col,0);

  /***** Drawing of letter 'L' *****/

  amove(X(31),Y(6));
  aline(X(31),Y(31));
  aline(X(35),Y(31));
  aline(X(35),Y(7));
  aarc(X(37),Y(7),X(37),Y(5),0);
  aline(X(53),Y(5));
  aline(X(53),Y(4));
  aline(X(33),Y(4));
  aarc(X(33),Y(6),X(31),Y(6),1);
  amove(X(33),Y(6));
  paint(col,0);

  /***** Drawing of letter 'S' *****/

  amove(X(55),Y(4));
  aline(X(75),Y(4));
  aarc(X(75),Y(6),X(77),Y(6),0);
  aline(X(77),Y(16));
  aarc(X(75),Y(16),X(75),Y(18),0);
  aline(X(61),Y(18));
  aarc(X(61),Y(20),X(59),Y(20),1);
  aline(X(59),Y(28));
  aarc(X(61),Y(28),X(61),Y(30),1);
  aline(X(77),Y(30));
  aline(X(77),Y(31));
  aline(X(57),Y(31));
  aarc(X(57),Y(29),X(55),Y(29),0);
  aline(X(55),Y(19));
  aarc(X(57),Y(19),X(57),Y(17),0);
  aline(X(71),Y(17));
  aarc(X(71),Y(15),X(73),Y(15),1);
  aline(X(73),Y(7));
  aarc(X(71),Y(7),X(71),Y(5),1);
  aline(X(55),Y(5));
  aline(X(55),Y(4));
  amove(X(57),Y(19));
  paint(col,0);
  amove(X(75),Y(6));
  paint(col,0);

  /***** Drawing of letter 'A' *****/

  amove(X(79),Y(4));
  aline(X(79),Y(29));
  aarc(X(81),Y(29),X(81),Y(31),1);
  aline(X(99),Y(31));
  aarc(X(99),Y(29),X(101),Y(29),1);
  aline(X(101),Y(4));
  aline(X(97),Y(4));
  aline(X(97),Y(17));
  aline(X(80),Y(17));
  aline(X(80),Y(4));
  aline(X(79),Y(4));
  amove(X(80),Y(18));
  aline(X(80),Y(28));
  aarc(X(82),Y(28),X(82),Y(30),1);
  aline(X(95),Y(30));
  aarc(X(95),Y(28),X(97),Y(28),1);
  aline(X(97),Y(18));
  aline(X(80),Y(18));
  amove(X(80),Y(30));
  dot();
  amove(X(99),Y(6));
  paint(col,0);

  amove(posx,posy);
}


/*******************************
 *                             *
 *       DEMO: PRESENTS        *
 *                             *
 *******************************/

/* Shows the first picture of the demo */

void presents()
{
  int direct;
  short prescount,y;

  drwmod(64);
  fscrn(XHR_LightWhite);

  amove(x_cent - ((unsigned short) x_cent * 54) / 130, y_cent + 80);
  ELSA(0, (double) x_cent/130);                            /* draw ELSA-Logo */

  colors(XHR_LightWhite,XHR_Black);
  amove(x_cent-3*4*16,240);
  stringout("proudly presents",3);

  delay(2.0);
  if(quit)
    return;

  colors(XHR_LightRed,XHR_Black);             /* draw 'XHR' with rising zoom */
  for (count=1; count <= 16; count++)
  { amove(x_cent-16*3*4,-2*16);
    stringout("XHR",count);
    delay(0.05);
    if(quit)
      return;
  };

  colors(XHR_LightWhite,XHR_LightWhite);
  amove(0,0);
  rfrct(x_width-1,4);
  colors(XHR_LightRed,XHR_Black);
  delay(MAXPRESENT);
}

/*******************************
 *                             *
 *       DEMO: FilledRects     *
 *                             *
 *******************************/


void FilledRects()
{
  XHR_color col;
  short rectcount;

  /***** erase screen with some effects *****/

  drwmod(64);
  colors(XHR_LightWhite,XHR_LightWhite);               /* color ==> XHR */
  amove(x_cent,y_cent);                                /* move to center */
  dot();                                               /* draw pixel */
  for (count=0; count<=x_width; count++)
  { amove(x_cent,y_cent+count);
    rline(count,-count);                               /* relative lines */
    rline(-count,-count);
    rline(-count,count);
    rline(count,count);
  };

  /***** draw filled rectangles at RANDOM positions *****/

  drwmod(0);
  for (rectcount = 0; rectcount<MAXRECTS; rectcount++) {
    col = RANDOM(256);                                 /* calc color */
    colors(col,col);                                   /* color ==> XHR */
    amove(RANDOM(x_width-140),RANDOM(y_width-100));    /* lowerleft corner */
    rfrct(RANDOM(100)+39,RANDOM(50)+49);               /* draw filled rect */
    if (!(rectcount & 0xff))
      if (kbhit())
      { quit=(getch()==ESCAPE);
        rectcount = MAXRECTS;
      }
  }
};

/*******************************
 *                             *
 *       DEMO: Circles         *
 *                             *
 *******************************/


void circles()
{
  XHR_color col;
  int cnt,circcount;

/***** Erase picture with some effects *****/

  drwmod(64);

  colors(XHR_LightBlue,XHR_LightBlue);
  for (count=0; count < x_width; count++)
  {
    amove(count,0);
    cnt = 0;
    do {
      rline(0,100);
      rmove(0,100);
      cnt += 200;
    } while (cnt <= y_width);
    amove(x_width-1-count,100);
    cnt = 0;
    do {
      rline(0,100);
      rmove(0,100);
      cnt += 200;
    } while (cnt <= y_width);
  }

  colors(XHR_LightWhite,XHR_LightWhite);
  amove(x_cent,y_cent);
  dot();
  for (count=1; count <= x_cent; count++)
  { amove(x_cent-count,y_cent-(int )((long )y_width*count/x_width));
    rrct(2*count,2*(int )((long )y_width*count/x_width));
  };


  /***** Draw circles *****/

  drwmod(67);
  amove(x_cent,y_cent);                      /* Move to center of basescreen */
  for (col=1, circcount=0; circcount<MAXCIRC; circcount++) {
    colors(col,col);                                         /* define color */
    for (count=0; count <= (y_width/2); crcl(count++))
    col = (col % 15)+1;                                        /* next color */
    if(kbhit())
    { quit=(getch()==ESCAPE);
      circcount=MAXCIRC;
    }
  }
}

/*******************************
 *                             *
 *       DEMO: Ellipses        *
 *                             *
 *******************************/

void ellipses()
{
  XHR_color col;
  short elpscount;

  /***** erase screen with some effects *****/

  drwmod(64);
  for (count=1; count<= x_cent; count++)
  { colors((count >> 4) % 16,(count >> 4) % 16);
    amove(x_cent-count,y_cent-count);
    rrct(count*2,count*2);
  };
  colors(XHR_Black,XHR_Black);
  amove(x_cent,y_cent);
  dot();
  for (count=1; count <= x_cent; count++)
  { amove(x_cent-count,y_cent-count);
    rrct(count*2,count*2);
  };


  /***** Draw ellipses *****/

  amove(x_cent,y_cent);                      /* Move to center of basescreen */
  col = XHR_LightWhite;
  for (elpscount = 0; elpscount<MAXELPS; elpscount++) {
    for (count=2; count <= (x_width / 8)-3; count++) {
      col = (col+1) % 256;
      if ((col % 16) == 0) colors(col / 16,col / 16);
      elps(10 + (count-2) % 8,10 - (count-2) % 8,count*4);
    };
    if (kbhit())
    { quit=(getch()==ESCAPE);
      elpscount = MAXELPS;
    }
  }
}

/*******************************
 *                             *
 *       DEMO: SWEEP           *
 *                             *
 *******************************/

void sweep()
{
  short count1, count2, col, x, y, sweepcount;

  /* erase screen with some effects */
  opmod( 0 );
  for( col = 1; col >= 0; col-- ) {
    colors( col, col );
    for( count1 = 0; count1 <= 32; count1++ ) {
      for( count2 = 0; count2 <= (x_width / 64) - 1; count2++ ) {
	amove( count2*64+32+count1, 0 );
	rline( 0,y_width );
	rmove( -2*count1, -y_width );
	rline( 0, y_width );
      }
    }
  }

  opmod(3);
  col=XHR_LightYellow;
  for (sweepcount = 0; sweepcount<MAXSWEEP; sweepcount++) {
    col = col % 15 + 1;
    colors(col,col);
    for(x=0; x<x_width; amove(0,0), aline(x++,y_width));
    col = col % 15 + 1;
    colors(col,col);
    for(y=y_width; y>=0; amove(0,0), aline(x_width,y--));
    col = col % 15 + 1;
    colors(col,col);
    for(y=0; y<y_width; amove(x_width-1,0), aline(0,y++));
    col = col % 15 + 1;
    colors(col,col);
    for(x=0; x<x_width; amove(x_width-1,0), aline(x++,y_width));
    if (kbhit())
    { quit=(getch()==ESCAPE);
      sweepcount=MAXSWEEP;
    }
  }
}


/*******************************
 *                             *
 *       DEMO: BaseZoom        *
 *                             *
 *******************************/

void BaseZoom()
{
  unsigned short zoomfac, maxzoomfac, zoomcount;
  unsigned short code;

  /* do this only on Gemini's with flickerfree hardware zoom features */
  code = rdini(-1);   /* read board code */

  if( code != 0x100 && code != 0x102 && code != 0x104 &&
      code != 0x106 && code != 0x108 && code != 0x10A &&
      code != 0x10C && code != 0x10E && code != 0x110 &&
      code != 0x112
    )
    return;

  maxzoomfac = 2;   /* zoom factor max 2 for all GEMINI boards */

  for (zoomcount=0; zoomcount<MAXZOOM; zoomcount++) {
    if(delay(1.0))
      zoomcount=MAXZOOM;
    for (zoomfac=2; zoomfac<=maxzoomfac; zoomfac++) {
      zoom(zoomfac,zoomfac);
      delay(0.07);
    }
    if(delay(1.0))
      zoomcount=MAXZOOM;
    for (zoomfac=maxzoomfac; zoomfac>=1; zoomfac--) {
      zoom(zoomfac,zoomfac);
      delay(0.07);
    }
  }
  zoom(1,1);
}

/*******************************
 *                             *
 *       DEMO: FilledArea      *
 *                             *
 *******************************/

static short LineTab1[] = {
  64,-64,64,64,64,-64,64,64,
  64,-64,64,64,64,-64,64,64,
  64,-64,64,64,64,-64,64,64,
  64,-64,64,64,64,-64,64,64,
  64,-64,64,64,64,-64,64,64,
  64,-64,64,64,64,-64,64,64,
  64,-64,64,64,64,-64,64,64
};

static short LineTab2[] = {
  270,220,270,290,320,290,320,240,
  370,240,370,320,410,320,410,220,
  500,320,550,190,550,190,550,300,
  590,300,590,160,490,160,510,230,
  390,110,480,110,480,70,360,70,
  360,20,310,20,310,80,200,200
};

void FilledArea()
{
  short count, areacount;

  /* erase picture with zigzag lines */

  drwmod(64);
  colors(XHR_LightYellow,XHR_LightYellow);
  for (count = 0; count <= y_width / 2 + 32; count++) {
    amove(0,64 + y_width - count);
    rpll(x_width / 64 + 1,(short far *) LineTab1);
    amove(0,count);
    rpll(x_width / 64 + 1,(short far *) LineTab1);
  }
  colors(XHR_Black,XHR_Black);
  for (count = y_width / 2 + 32; count>=0; count--) {
    amove(0,64 + y_width - count);
    rpll(x_width / 64 + 1,(short far *) LineTab1);
    amove(0,count);
    rpll(x_width / 64 + 1,(short far *) LineTab1);
  }
  opmod(0);


  /* perform filling of closed area */

  deforg(x_cent - 320, y_cent - 200);
  amove(200, 200);                                                /* move CP */
  colors(XHR_LightWhite,XHR_LightWhite);           /* XHR_LightWhite ==> XHR */
  apll(24,(short far *) LineTab2);                      /* absolute polyline */
  amove(300,200);                                                 /* move CP */
  colors(XHR_Red,XHR_Red);                                    /* Red ==> XHR */
  paint(XHR_LightWhite,0);                               /* fill closed area */
  deforg(0,0);                               /* repair origin */

  delay(MAXAREA);
}

/*******************************
 *                             *
 *       DEMO: Cake            *
 *                             *
 *******************************/

void Cake()
{
  short x, y, i, xrect, yrect, erasedcount, cakecount;
  unsigned char erased[16][16];

  /* erase screen with RANDOM rectangles */

  for(xrect=x_width/16; 16*xrect < x_width; xrect++);
  for(yrect=y_width/16; 16*yrect < y_width; yrect++);

  for(x=0; x<16; x++)
    for(y=0; y<16; y++) erased[x][y]=0;
  erasedcount = 0;
  colors(0,0);
  drwmod(64);
  do {
    x=RANDOM(16);
    y=RANDOM(16);
    if (!erased[x][y]) {
      amove(x*xrect,y*yrect);
      rfrct(xrect-1,yrect-1);
      erased[x][y]=1;
      erasedcount++;
    }
  } while (erasedcount < 256);

  /* draw outlines of cake */

  colors(XHR_LightWhite,XHR_LightWhite);                   /* colors ==> XHR */
  amove(x_cent,y_cent);                                           /* move CP */
  aline(x_cent+176,y_cent+88);                              /* absolute line */
  aearc(4,1,x_cent,y_cent,x_cent+241,y_cent-32,0);      /* abs. ellipses arc */
  aline(x_cent,y_cent);
  amove(x_cent-250,y_cent-50);
  aearc(4,1,x_cent,y_cent-50,x_cent+241,y_cent-82,0);
  aline(x_cent+241,y_cent-32);
  amove(x_cent-250,y_cent);
  aline(x_cent-250,y_cent-50);
  amove(x_cent+50,y_cent+10);
  aline(x_cent+226,y_cent+98);
  aearc(4,1,x_cent+50,y_cent+10,x_cent+291,y_cent-22,1);
  aline(x_cent+50,y_cent+10);
  rline(0,-17);                                             /* relative line */
  amove(x_cent+298,y_cent+10);
  rline(0,-50);
  aearc(4,1,x_cent+50,y_cent-40,x_cent+291,y_cent-72,1);
  rline(0,50);
  amove(x_cent+291,y_cent-72);
  rline(-50,7);
  amove(x_cent+176,y_cent+88);
  rline(0,-16);

  colors(XHR_LightBlue,XHR_LightBlue);
  amove(x_cent-10,y_cent-10);
  paint(XHR_LightWhite,0);                        /* fill closed area  */
  colors(XHR_LightCyan,XHR_LightCyan);
  amove(x_cent,y_cent-130);
  paint(XHR_LightWhite,0);
  colors(XHR_Cyan,XHR_Cyan);                                 /* Cyan ==> XHR */
  amove(x_cent+40,y_cent);
  paint(XHR_LightWhite,0);                     /* fill closed area with cyan */
  colors(XHR_LightRed,XHR_LightRed);
  amove(x_cent+100,y_cent+10);
  paint(XHR_LightWhite,0);                        /* fill closed area */
  amove(x_cent+52,y_cent);
  colors(XHR_LightGreen,XHR_LightGreen);           /* XHR_LightGreen ==> XHR */
  paint(XHR_LightWhite,0);           /* fill closed area with XHR_LightGreen */
  amove(x_cent+293,y_cent-47);
  colors(XHR_Green,XHR_Green);                              /* Green ==> XHR */
  paint(XHR_LightWhite,0);                    /* fill closed area with green */

  delay(MAXCAKE);
 }


